/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLDEVICE_H__
#define __GGLDEVICE_H__

#include <glib.h>
#include <ggl/gglbase.h>
#include <ggl/ggltimeout.h>
#include <ggl/gglvisual.h>
#include <ggl/gglobserver.h>

#define GGL_DEVICE(device) (GGL_CAST(device,GglDevice*,GGLDEVICE))

/********** IMPORTANT ********************/
/* You can have up to 32 kinds of events */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct {
  GglObserver *observer;
  GglEventMask mask;
} GglDeviceItem; /* The elements of the list are pointers to this structure */

typedef struct _GglDevice GglDevice;

struct _GglDevice{
  GglBase base;
  int id;
  GList *observers;	/* The list of observers to be notified */
  
  GglTimeout *timeout; /* For uninstall later */

  /* this method is invoked x times per second */
  void (*ticker)(GglDevice *device); 
};

typedef void (*GglDevCallBack)(GglDeviceItem *observer, void *data);

/* Constructors */
GglDevice*	ggl_device_new		(void);

void		ggl_device_init		(GglDevice *device);

void		ggl_device_dispose	(GglDevice *device);

void		ggl_device_finalize	(GglDevice *device);

void		ggl_device_attach	(GglDevice *device,
					 GglObserver *observer);

int		ggl_device_detach	(GglDevice *device,
					 GglObserver *observer);

int		ggl_device_set_mask	(GglDevice *device,
					 GglObserver *observer,
					 GglEventMask mask);

void		ggl_device_install	(GglDevice *device,
					 GglTimeout *timeout);

void 		ggl_device_foreach_observer (GglDevice *device,
					 GglDevCallBack callback,
					 void *data);


#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLDEVICE_H__ */
