/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLFRAME_H__
#define __GGLFRAME_H__

#include <glib.h>
#include <ggl/gglbase.h>
#include <ggl/gglvisual.h>
#include <ggl/gglmask.h>

#define GGL_FRAME(frame) (GGL_CAST(frame,GglFrame*,GGLFRAME))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglFrame GglFrame;

struct _GglFrame{
  GglBase base;
  int id;
  GglVisual *img;	/* Image of the frame        */
  GglColor *transparent;/* Color used for transparency */
  unsigned short width,height; /* Dimensions		 */
  short hook_x, hook_y;   /* Hook of the frame         */
  short action_x,action_y;/* Action point of the frame */
};

/* Constructors */
GglFrame*	ggl_frame_new		(GglVisual *img);

void 		ggl_frame_init		(GglFrame *frame,
					 GglVisual *img);

void 		ggl_frame_dispose	(GglFrame *frame);
                                         
void 		ggl_frame_finalize	(GglFrame *frame);
                                         
GglColor*	ggl_frame_get_transparent(GglFrame *frame);

void	        ggl_frame_set_transparent(GglFrame *frame,
					 GglColor *transparent);

void		ggl_frame_get_hook	(GglFrame *frame,
					 short *x,
					 short *y);

void		ggl_frame_set_hook	(GglFrame *frame,
					 short x,
					 short y);

void		ggl_frame_get_action	(GglFrame *frame,
					 short *x,
					 short *y);

void		ggl_frame_set_action	(GglFrame *frame,
					 short x,
					 short y);

/* Main methods for drawing a frame */
void		ggl_frame_draw		(GglFrame *frame,
					 GglVisual *visual,
					 int x,
					 int y);

void		ggl_frame_get_size	(GglFrame *frame,
					 int *width,
					 int *height);

int 		ggl_frame_collision	(GglFrame *frame1,
					 GglFrame *frame2,
					 int x1,
					 int y1,
					 int x2,
					 int y2,
					 float *angle);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLFRAME_H__ */
