/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* Interface with GGI Graphic context
 */

#ifndef __GGLGC_H__
#define __GGLGC_H__

#include <ggl/gglcolor.h>
#include <ggl/gglvisual.h>
#include <ggi/ggi.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

int 		ggl_gc_set_foreground	(GglVisual *visual,
					 GglColor *fg);
    
int 		ggl_gc_get_foreground	(GglVisual *visual,
					 GglColor *fg);

int		ggl_gc_set_background	(GglVisual *visual,
					 GglColor *bg);

int 		ggl_gc_get_background	(GglVisual *visual,
					 GglColor *bg);

int		ggl_gc_get_clipping 	(GglVisual *visual,
					 int *x0,
					 int *y0,
					 int *x1,
					 int *y1);

int		ggl_gc_set_clipping 	(GglVisual *visual,
					 int x0,
					 int y0,
					 int x1,
					 int y1);

int 		ggl_gc_draw_pixel	(GglVisual *visual, 
					 int x, 
					 int y);
int 		ggl_gc_put_pixel 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 GglColor *color);
int 		ggl_gc_get_pixel 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 GglColor *color);

int 		ggl_gc_draw_hline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width);
int 		ggl_gc_put_hline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 void *buffer);
int 		ggl_gc_get_hline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 void *buffer);

int 		ggl_gc_draw_vline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int height);
int 		ggl_gc_put_vline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int height, 
					 void *buffer);
int 		ggl_gc_get_vline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int height, 
					 void *buffer);

int 		ggl_gc_draw_box 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 int height);
					 
int 		ggl_gc_put_box 		(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 int height, 
					 void *buffer);

int 		ggl_gc_get_box 		(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 int height, 
					 void *buffer);

int 		ggl_gc_fill_visual 	(GglVisual *visual);

int 		ggl_gc_draw_line 		(GglVisual *visual,
					 int x1, 
					 int y1, 
					 int x2, 
					 int y2);

int 		ggl_gc_copy_box 	(GglVisual *visual, 
					 int x0, 
					 int y0, 
					 int width, 
					 int height, 
					 int x1, 
					 int y1);

int 		ggl_gc_cross_blit 	(GglVisual *source, 
					 int source_x, 
					 int source_y, 
					 int source_width, 
					 int source_height, 
					 GglVisual *target, 
					 int target_x, 
					 int target_y);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLGC_H__ */
