/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLGRAPHIC_H__
#define __GGLGRAPHIC_H__

#include <glib.h>
#include <ggl/gglrectangle.h>
#include <ggl/gglobserver.h>

#define GGL_GRAPHIC(graphic) (GGL_CAST(graphic,GglGraphic*,GGLGRAPHIC))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglGraphic GglGraphic;

struct _GglGraphic{
    GglObserver observer; /* Each graphic class can be controlled by input system */
    int id;
    char modified; /* modified flag */
    char visible;  /* visible flag */
    float x,y,z;   /* Coordinates */

    char m_visible; /* temporary visible flag (user modifications goes here) */
    float m_x,m_y,m_z; /* temporary coordinates (user modifications goes here) */
    int user_modif;    /* Flag to mark user modifications */

    void (*draw)        (GglGraphic *graphic, GglVisual *visual,
			 int ofs_x, int ofs_y, GList *overlaps);
    int  (*change)      (GglGraphic *graphic);
    void (*get_area)    (GglGraphic *graphic, GglRectangle *area);
    void (*set_visible) (GglGraphic *graphic, int visible);
};

#define MODIF_VISIBLE  0x01
#define MODIF_POSITION 0x02

void		ggl_graphic_init	(GglGraphic *graphic);

void		ggl_graphic_finalize	(GglGraphic *graphic);

void		ggl_graphic_draw        (GglGraphic *graphic,
					 GglVisual *visual,
					 int ofs_x,
					 int ofs_y,
					 GList *overlaps);

int             ggl_graphic_change      (GglGraphic *graphic);

int             ggl_graphic_has_changed (GglGraphic *graphic);

void            ggl_graphic_get_area    (GglGraphic *graphic,
					 GglRectangle *area);

void            ggl_graphic_set_visible (GglGraphic *graphic,
					 int visible);

int             ggl_graphic_get_visible (GglGraphic *graphic);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLGRAPHIC_H__ */

