/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* ChangeLog for the video Stuff
 *
 *    02/22/1999
 *     - I begin the REAL work }:-)
 *     - The gglvideo.h has been translated to English (or something like
 *       it :)
 *     - This ChangeLog and the TODO section :)
 *     - Changed the screen coordinates to int (coherently to other
 *       classes in GGL.
 *
 * Things I have TODO:
 *     - Implementation of FLIs and raw AVIs.
 *     - In the far future MPEG or any other block-stream highly-compressed
 *       video format (perhaps my own algorithm?)
 */



#ifndef __GGLVIDEO_H__
#define __GGLVIDEO_H__

#include <glib.h>
#include <ggl/gglbase.h>
#include <ggl/gglscreen.h>

#define GGL_VIDEO(video) (GGL_CAST (video, GglVideo*, GGLVIDEO))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglVideo GglVideo;

/* Notes about data structures:
 * - In the object we have a file reference (the video stream), and
 *   play status (current frame, h/v size, etc).
 * - The maximum length of a video file is in a 32 bits unsigned integer. So
 *   you will be limited to 4 GB in your little videos. I can say 4 GB
 *   will be enough (like the hell Gates), but the DVD can make this be
 *   some kind of obsolete stand. If anyone complains, we can use an
 *   unsigned long long ;)
 * - It's an abstract class. The idea is hanging FLIs and AVIs from this one,
 *   and (who knows) MPEG-1 or MPEG-2 later.
 * - This object definition works for a frame based video type. Including
 *   MPEG or other blockstream-based video types means another definition.
 *   The best solution will probably be a different parent class.
 */

struct _GglVideo{
  GglBase base;
  int id;

  FILE *file;                     /* File ID */
  GglScreen *screen;		/* Framebuffer */

  unsigned long numframes;      /* Number of frames in the video */
  unsigned long currentframe;   /* Current frame */

  int h_size;			/* H/V size of the video */
  int v_size;

};

/* Constructors */
GglVideo*	ggl_video_new		();

/* Init all the internal structures */
void		ggl_video_init		(GglVideo *video);

/* File Methods */
void		ggl_video_open		(GglVideo *video,
					 char *fname);
void		ggl_video_close		(GglVideo *video);
void		ggl_video_attach	(GglVideo *video,
					 FILE *file_id);
void		ggl_video_release	(GglVideo *video);
/* Display methods */
void		ggl_video_attach_screen	(GglVideo *video,
					 GglScreen *screen,
					 int x, int y);
void		ggl_video_release_screen (GglVideo *video);

/* Play methods */
void		ggl_video_play		(GglVideo *video);
void		ggl_video_play_segment	(GglVideo *video,
					 unsigned long frame_ini,
					 unsigned long frame_fin);

/* Frame selection methods */
void		ggl_video_rewind	(GglVideo *video);
void		ggl_video_go_to		(GglVideo *video,
					 unsigned long frame);
void		ggl_video_next		(GglVideo *video);
void		ggl_video_previous	(GglVideo *video);

/* Single frame draw */
void		ggl_video_draw_frame	(GglVideo *video);

/* Property methods */
unsigned long	ggl_video_frames	(GglVideo *video);
char		ggl_video_end		(GglVideo *video);
unsigned long	ggl_video_current	(GglVideo *video);
int		ggl_video_size_h	(GglVideo *video);
int		ggl_video_size_v	(GglVideo *video);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLVIDEO_H__ */

