/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* Interface with GGI
 */

#ifndef __GGLVISUAL_H__
#define __GGLVISUAL_H__

#include <ggl/gglpalette.h>
#include <ggi/ggi.h>

#define GGL_VISUAL(gvisual) (GGL_CAST(gvisual,GglVisual*,GGLVISUAL))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglVisual GglVisual;

struct _GglVisual{
  GglBase base;         /* Base class */
  int id;               /* Class Identifier */
  ggi_visual_t visual;  /* GGI Visual*/
  int width,height;     /* Visual Size*/
  int swidth,sheight;   /* Visible window size (for scroll)*/
  int scrollx,scrolly;  /* Visible window position (scroll) */
  int scrolled;         /* Flag that indicates that scroll changed*/
  GglDepth depth;	/* Number of bits/pixel */
  int frames;           /* Number of frames (for multiple buffering) */
  unsigned short flags; /* Flags of the visual */
};

/* FLAGS */

#define GGL_VISUAL_IS_MEMORY 0x0001

/* Constructors */
GglVisual*	ggl_visual_new          ();

void		ggl_visual_init		(GglVisual *visual);

void		ggl_visual_dispose	(GglVisual *visual);

void		ggl_visual_finalize	(GglVisual *visual);

GglVisual*      ggl_visual_new_target   (char *target);

void		ggl_visual_init_target	(GglVisual *visual,
					 char *target);

int             ggl_visual_set_mode     (GglVisual *visual,
					 int width,
					 int height,
					 int vwidth,
					 int vheight,
					 GglDepth depth,
					 int frames);
int             ggl_visual_check_mode     (GglVisual *visual,
					   int width,
					   int height,
					   int vwidth,
					   int vheight,
					   GglDepth depth,
					   int frames);
int             ggl_visual_set_origin   (GglVisual *visual,
                                         int x,
                                         int y);
int		ggl_visual_flush	(GglVisual *visual);
int		ggl_visual_flush_region	(GglVisual *visual,
					 int x,
					 int y,
					 int w,
					 int h);
					 
void 		ggl_visual_get_palette	(GglVisual *visual,
					 GglPalette *palette);

void 		ggl_visual_set_palette	(GglVisual *visual,
					 GglPalette *palette);

int 		ggl_visual_get_bits_size(GglVisual *visual);

int             ggl_visual_set_disp_frame (GglVisual *visual,
					   int frame);
int             ggl_visual_set_read_frame (GglVisual *visual,
					   int frame);
int             ggl_visual_set_write_frame (GglVisual *visual,
					    int frame);

int             ggl_visual_get_disp_frame (GglVisual *visual);

int             ggl_visual_get_read_frame (GglVisual *visual);

int             ggl_visual_get_write_frame (GglVisual *visual);

void* 		ggl_visual_get_read_pointer (GglVisual *visual);

void*		ggl_visual_get_write_pointer (GglVisual *visual);

void* 		ggl_visual_get_read_pointer_nth(GglVisual *visual,
					 	int frame);

void*		ggl_visual_get_write_pointer_nth(GglVisual *visual,
					 	int frame);

/* only for memory visuals! */
					 
int             ggl_visual_change_depth(GglVisual *visual,
                                         GglDepth target);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLVISUAL_H__ */
