/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglmousedev.h>
#include <stdlib.h>

void gglmousedev_ticker(GglMouseDev *mousedev);

void ggl_mousedev_init(GglMouseDev *mousedev, GglVisual *visual)
{
    ggl_device_init(GGL_DEVICE(mousedev));
    mousedev-> id=GGLMOUSEDEV;
    mousedev-> visual=visual;
//    ggiAddEventMask(mousedev->visual,emPointer);
    GGL_DEVICE(mousedev)->ticker=(void(*)(GglDevice*))gglmousedev_ticker;
}

GglMouseDev* ggl_mousedev_new(GglVisual *visual)
{
    GglMouseDev *mousedev=(GglMouseDev *)malloc(sizeof(GglMouseDev));
    ggl_base_set_size(GGL_BASE(mousedev),sizeof(GglMouseDev));
    
    ggl_mousedev_init(mousedev,visual);
    
    return mousedev;
}

void ggl_mousedev_finalize(GglMouseDev *mousedev)
{
    ggl_device_finalize(GGL_DEVICE(mousedev));
    ggiRemoveEventMask(mousedev->visual, emPointer);
}

void ggl_mousedev_dispose(GglMouseDev *mousedev)
{
    ggl_mousedev_finalize(mousedev);
    free(mousedev);
}

void gglmousedev_button_notify(GglDeviceItem *devitem, ggi_event *event)
{
    if (devitem == NULL) g_warning("gglmousedev_button_notify: NULL pointer\n");
    switch (event->any.type)
    {
    case evPtrButtonPress:
    case evPtrButtonRelease:
	if (devitem->mask & GGL_EVMASK_BUTTON)
	    ggl_observer_update_button(devitem->observer,GGL_MOUSE,0,
				       event->pbutton.button,
				       (event->any.type==evPtrButtonPress));
	break;
    case evPtrAbsolute:
	if (devitem->mask & GGL_EVMASK_AXIS)
	{
	    ggl_observer_update_axis(devitem->observer,GGL_MOUSE,0,
				     0,event->pmove.x);
	    ggl_observer_update_axis(devitem->observer,GGL_MOUSE,0,
				     1,event->pmove.y);
	    ggl_observer_update_axis(devitem->observer,GGL_MOUSE,0,
				     2,event->pmove.z);
	    ggl_observer_update_axis(devitem->observer,GGL_MOUSE,0,
				     3,event->pmove.wheel);
	}
	break;
    }
}

void gglmousedev_ticker(GglMouseDev *mousedev)
{
    GglVisual *visual=mousedev->visual;
    ggi_event event;
    struct timeval t;
    int err;
    t.tv_sec=0;
    t.tv_usec=0;
    if ((err=ggiEventPoll(visual->visual,emPointer,&t))>0)
    {
	ggiEventRead(visual->visual,&event,emPointer);
	ggl_device_foreach_observer(GGL_DEVICE(mousedev),
				    (GglDevCallBack)gglmousedev_button_notify,
				    &event);
    }else{
	if (err<0)
	    g_warning("GGLMOUSEDEV: error code <%i>",err);
    }
}
