/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglgc.h>
#include <stdlib.h>

int 		ggl_gc_set_foreground	(GglVisual *visual,
					 GglColor *fg)
{
    return ggiSetGCForeground(visual->visual, fg->color);
}
    
int 	        ggl_gc_get_foreground	(GglVisual *visual,
					 GglColor *color)
{
    return ggiGetGCForeground(visual->visual,&color->color);
}

int 		ggl_gc_set_background	(GglVisual *visual,
					 GglColor *fg)
{
    return ggiSetGCBackground(visual->visual, fg->color);
}

int 	        ggl_gc_get_background	(GglVisual *visual,
					 GglColor *fg)
{
    return ggiGetGCBackground(visual->visual, &fg->color);
}

int		ggl_gc_get_clipping 	(GglVisual *visual,
					 int *x0,
					 int *y0,
					 int *x1,
					 int *y1)
{
    return ggiGetGCClipping(visual->visual,x0,y0,x1,y1);
}

int		ggl_gc_set_clipping 	(GglVisual *visual,
					 int x0,
					 int y0,
					 int x1,
					 int y1)
{
    return ggiSetGCClipping(visual->visual,x0,y0,x1,y1);
}

int 		ggl_gc_draw_pixel	(GglVisual *visual, 
					 int x, 
					 int y)
{
    return ggiDrawPixel(visual->visual,x,y);
}

int 		ggl_gc_put_pixel 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 GglColor *color)
{
    return ggiPutPixel(visual->visual,x,y,color->color);
}

int 		ggl_gc_get_pixel 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 GglColor *color)
{
    return ggiGetPixel(visual->visual,x,y,&color->color);
}

int 		ggl_gc_draw_hline 	(GglVisual *visual,
					 int x, 
					 int y, 
					 int width)
{
    return ggiDrawHLine(visual->visual,x,y,width);
}

int 		ggl_gc_put_hline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 void *buffer)
{
    return ggiPutHLine(visual->visual,x,y,width,buffer);
}

int 		ggl_gc_get_hline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 void *buffer)
{
    return ggiGetHLine(visual->visual,x,y,width,buffer);
}

int 		ggl_gc_draw_vline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int height)
{
    return ggiDrawVLine(visual->visual,x,y,height);
}

int 		ggl_gc_put_vline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int height, 
					 void *buffer)
{
    return ggiPutVLine(visual->visual,x,y,height,buffer);
}

int 		ggl_gc_get_vline 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int height, 
					 void *buffer)
{
    return ggiGetVLine(visual->visual,x,y,height,buffer);
}

int 		ggl_gc_draw_box 	(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 int height)
{
    return ggiDrawBox(visual->visual,x,y,width,height);
}
					 
int 		ggl_gc_put_box 		(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 int height, 
					 void *buffer)
{
    return ggiPutBox(visual->visual,x,y,width,height,buffer);
}

int 		ggl_gc_get_box 		(GglVisual *visual, 
					 int x, 
					 int y, 
					 int width, 
					 int height, 
					 void *buffer)
{
    return ggiGetBox(visual->visual,x,y,width,height,buffer);
}

int 		ggl_gc_fill_visual 	(GglVisual *visual)
{
    return ggiFillscreen(visual->visual);
}

int 		ggl_gc_draw_line 	(GglVisual *visual,
					 int x1, 
					 int y1, 
					 int x2, 
					 int y2)
{
    return ggiDrawLine(visual->visual,x1,y1,x2,y2);
}

int 		ggl_gc_copy_box 	(GglVisual *visual, 
					 int x0, 
					 int y0, 
					 int width, 
					 int height, 
					 int x1, 
					 int y1)
{
    return ggiCopyBox(visual->visual,x0,y0,width,height,x1,y1);
}

/*********************************************************************/
void gglgc_own_blit(GglVisual *source,int x, int y, int w, int h,
			GglVisual *target, int dx, int dy)
{
    int i,srcpos,dstpos,srcinc,dstinc,realwidth;
    int bits_size=(ggl_visual_get_bits_size(target))>>3; /* 8 bits -> 1 byte */
    char *src,*dst;
    src=ggl_visual_get_read_pointer(source);
    dst=ggl_visual_get_write_pointer(target);
    srcpos=(x+y*source->swidth)*bits_size;
    dstpos=(dx+dy*target->swidth)*bits_size;
    srcinc=(source->swidth)*bits_size;
    dstinc=(target->swidth)*bits_size;
    realwidth=w*bits_size;
    for (i=0;i<h;i++)
    {
	memcpy(dst+dstpos,src+srcpos,realwidth);
	srcpos+=srcinc;
	dstpos+=dstinc;
    }
}

int ggl_gc_cross_blit(GglVisual *visual,int x, int y, int w, int h, GglVisual *dst,int dx, int dy)
{
    return ggiCrossBlit(visual->visual,x,y,w,h,dst->visual,dx,dy);
    //gglgc_own_blit(visual,x,y,w,h,dst,dx,dy);
}

