/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <ggl/ggllayer.h>

void ggl_layer_init(GglLayer *layer)
{
    layer-> id       = GGLLAYER;
    layer-> graphics = NULL;
    layer-> scroll_x = 0;
    layer-> scroll_y = 0;
    layer-> deleted  = NULL;
    layer-> draw     = NULL;
    layer-> change   = NULL;
}

GglLayer* ggl_layer_new()
{
    GglLayer *layer=(GglLayer*)malloc(sizeof(GglLayer));

    ggl_base_set_size(GGL_BASE(layer),sizeof(GglLayer));
    ggl_layer_init(layer);

    return layer;
    
}

void ggl_layer_finalize(GglLayer *layer)
{
    g_list_free(layer->graphics);
    g_list_free(layer->deleted);
}

void ggl_layer_dispose(GglLayer *layer)
{
    ggl_layer_finalize(layer);
}


static GList* graphic_rectangles_overlap(GglGraphic *gr, GList *modified)
{
    GList *res=NULL;
    GList *item;
    GglRectangle r1;
    ggl_graphic_get_area(gr,&r1);
    for (item=modified;item!=NULL;item=item->next)
    {
	GglRectangle *r2=(GglRectangle*)item->data;
	if (ggl_rectangle_overlap(r1,*r2))
	{
	    res=g_list_append(res,r2);
	}
    }
    return res;
}

void ggl_layer_draw(GglLayer *layer, GglVisual *visual, GList *modified)
{
    GList *item;
    GList *overlaps=NULL;
    for (item=layer->graphics;item!=NULL;item=item->next)
    {
	GglGraphic *graphic=(GglGraphic*)item->data;
	if ((overlaps=graphic_rectangles_overlap(graphic,modified))!=NULL)
	{
	    ggl_graphic_draw(graphic,visual,layer->scroll_x,layer->scroll_y,overlaps);
	    g_list_free(overlaps);
	    /* There is no need of freeing rectangles, because they are the
	     same (pointers) than "modified" ones, and they are freed by
	     GglScreen. */
	}
    }
}

void ggl_layer_change (GglLayer *layer, GList* *modified)
{
    GList *item;
    for (item=layer->graphics;item!=NULL;item=item->next)
    {
	GglGraphic *graphic=(GglGraphic*)item->data;
	GglRectangle *r=(GglRectangle*)malloc(sizeof(GglRectangle));
	ggl_graphic_get_area(graphic,r);
	if (ggl_graphic_change(graphic))
	{
	    /* If it changes, then add it's previous area and new area
	     * to the list
	     */
	    *modified=g_list_append(*modified,r);
	    r=(GglRectangle*)malloc(sizeof(GglRectangle));
	    ggl_graphic_get_area(graphic,r);
	    *modified=g_list_append(*modified,r);
	}else{
	    free(r);
	}
    }
    *modified=g_list_concat(*modified,layer->deleted);
    layer->deleted=NULL;
}

void ggl_layer_add (GglLayer *layer, GglGraphic *gr)
{
    layer->graphics=g_list_append(layer->graphics,gr);
}

void ggl_layer_remove(GglLayer *layer, GglGraphic *gr)
{
    GglRectangle *r=(GglRectangle*)malloc(sizeof(GglRectangle));
    ggl_graphic_get_area(gr,r);
    // Inserting this rectangle on deleted list to take it into account later
    layer->deleted=g_list_append(layer->deleted,r);
    layer->graphics=g_list_remove(layer->graphics,gr);
}
