/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglfli.h>
#include <stdlib.h>

#define GGL_FLI_DFL_FRAMES 30
#define GGL_FLI_MAGIC 0xaf11

/* Constructors */
void ggl_fli_init (GglFli *fli)
{
        ggl_video_init(GGL_VIDEO(fli->video));

	fli->id = GGLFLI;
	fli->fps = GGL_FLI_DFL_FRAMES;

	fli->fsize = 0;
	fli->offset = 0;
}

/* Init all the internal structures (format dependant - forbidden) */
GglFli * ggl_fli_new (void)
{
	GglFli *fli=(GglFli *)malloc(sizeof(GglFli));
	ggl_base_set_size(GGL_BASE(fli),sizeof(GglFli));

	ggl_fli_init(fli);

	return fli;
}

/* Fli format interpretation */

void ggl_fli_putonstart (GglFli *fli)
{
	rewind(fli->file);
	fli->offset = 0;
}

void ggl_fli_setoffset (GglFli *fli,int movement)
{
	fseek(fli->file,movement,SEEK_SET);
	offset = movement;
}

int ggl_fli_read (GglFli *fli, int size, void *buffer)
{
	if (-1 == fread(buffer,size,1,fli->file))
		g_print("libggl:ggl_fli_read::%s",g_strerror(errno));
	else
		offset += size;
}

void ggl_fli_read_header (GglFli *fli)
{
	short int sbuf;

	ggl_fli_putonstart(fli);
	ggl_fli_read(fli,4,&(fli->fsize));
	ggl_fli_read(fli,2,&sbuf);
	if (sbuf != GGL_FLI_MAGIC) {
		g_print("libggl:ggl_fli_read_header:Bad FLI header");
		return;
	}
	ggl_fli_read(fli,2,&sbuf);
	(GGLVIDEO(fli))->numframes = sbuf;
	ggl_fli_read(fli,2,&sbuf);
	(GGLVIDEO(fli))->h_size = sbuf;
	ggl_fli_read(fli,2,&sbuf);
	(GGLVIDEO(fli))->v_size = sbuf;
	ggl_fli_read(fli,4,NULL); /* Ignores color depth and flags fields */
	ggl_fli_read(fli,2,&sbuf);
	fli->fps = sbuf;
	ggl_fli_read(fli,8,NULL); /* Ignores next and frit fields */
	ggl_fli_read(fli,102,NULL); /* Ignores expand field */
	
}

void ggl_fli_read_frame (GglFli *fli)
{

}

/*
 * File Methods (forbidden video methods - see particular format
 * implementations)
 */
void ggl_fli_open (GglFli *fli, char *fname)
{
	ggl_video_open (GGLVIDEO(fli),fname);
	ggl_fli_read_header (fli);
}

void ggl_fli_close (GglFli *video)
{
	ggl_video_close (GGLVIDEO(fli),fname);
}

void ggl_fli_attach (GglFli *fli, FILE *file_id)
{
	ggl_video_attach(GGLVIDEO(fli),file_id);
	ggl_fli_read_header (fli);
}

void ggl_fli_release (GglFli *fli)
{
	ggl_video_release(GGLVIDEO(fli));
}

/* Display methods */
void ggl_fli_attach_screen (GglFli *fli, GglScreen *screen, int x, int y)
{
	ggl_video_attach_screen(GGLVIDEO(fli),screen,x,y);
}

void ggl_fli_release_screen (GglFli *fli)
{
	ggl_video_release_screen(GGLVIDEO(fli));
}

/* Play methods */
void ggl_fli_play (GglFli *fli)
{
	ggl_video_play(GGLVIDEO(fli));
}

void ggl_fli_play_segment (GglFli *fli, unsigned long frame_ini,
			     unsigned long frame_fin);
{
	ggl_video_play_segment(GGLVIDEO(fli),frame_ini,frame_fin);
}

/* Frame selection methods */
void ggl_fli_rewind (GglFli *fli)
{
	ggl_video_rewind (GGLVIDEO(fli));
	ggl_fli_setoffset (fli,128);

}

void ggl_fli_goto (GglFli *fli, unsigned long frame)
{
	ggl_video_goto(GGLVIDEO(fli),frame);
}

void ggl_fli_next (GglFli *fli)
{
	ggl_video_next(GGLVIDEO(fli));
}

void ggl_fli_previous	(GglFli *fli)
{
	ggl_video_previous(GGLVIDEO(fli));
}

/* Single frame draw (format dependant) */
void ggl_fli_draw_frame (GglFli *fli)
{
	ggl_video_draw_frame (GGLVIDEO(fli));
}

/* Property methods */
unsigned long ggl_fli_frames (GglFli *fli)
{
	return(ggl_video_frames (GGLVIDEO(fli)));
}

char ggl_fli_end (GglFli *fli)
{
	return (ggl_video_end (GGLVIDEO(fli)));
}

unsigned long ggl_fli_current (GglFli *video)
{
	return (ggl_video_current(GGLVIDEO(fli)));
}

int ggl_fli_size_h (GglFli *fli)
{
	return (ggl_video_size_h(GGLVIDEO(fli)));
}

int ggl_fli_size_v (GglFli *fli)
{
	return (ggl_video_size_v(GGLVIDEO(fli)));
}

